'use server';

import { requestGetFetch, requestPostFetch } from "@/utils/fetch";
import { getServerSession, Session } from "next-auth";
import { nextauthOptions } from "../nextauth-options";
import { SearchParams } from "@/types/searchParams";

export async function getAttempts(searchParams?: SearchParams) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const paramsArray = searchParams && Object.entries(searchParams);

    const queryString = paramsArray && paramsArray.map(([key, value]) => `${key}=${value}`).join('&');

    const path = 'v1/attempts' + `${queryString ? `?${queryString}` : ''}`;
    const response = await requestGetFetch({
      path,
      options: {
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        cache: "no-store",
      },
    });
    return response.data || [];
  } catch (error) {
    throw error;
  }
}

export async function deleteAttempt(id: number) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = `v1/attempts/${id}`;
    const response = await requestGetFetch({
      path,
      options: {
        method: "DELETE",
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        cache: 'no-store',
      },
    });
    return response;
  } catch (error) {
    throw error;
  }
}

interface CreateAttemptProps {
  name: string;
  permissions: string[];
}

export async function createAttempt({
  name,
  permissions,
}: CreateAttemptProps) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const response = await requestPostFetch({
      path: `v1/attempts`,
      data: {
        name,
        permissions,
      },
      token: token,
    });
    return response;
  } catch (error: any) {
    throw new Error(error.message);
  }
}

export async function storeAttempt(payload: any) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const response = await requestPostFetch({
      path: `v1/attempts`,
      data: { ...payload },
      token: token,
    });
    return response;
  } catch (error: any) {
    throw new Error(error.message);
  }
}

export async function editAttempt(id: number, payload: any,) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = `v1/attempts/${id}`;

    const response = await requestGetFetch({
      path,
      options: {
        method: "PUT",
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        body: JSON.stringify({ ...payload }),
        cache: 'no-store',
      },
    });
    return response;
  } catch (error: any) {
    throw new Error(error.message);
  }
}

interface UpdateAttemptProps {
  id: number;
  name: string;
  permissions: string[];
}

export async function updateAttempt({
  id,
  name,
  permissions,
}: UpdateAttemptProps) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = `v1/attempts/${id}`;

    const response = await requestGetFetch({
      path,
      options: {
        method: "PUT",
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        body: JSON.stringify({
          name,
          permissions,
        }),
        cache: 'no-store',
      },
    });
    return response;
  } catch (error: any) {
    throw new Error(error.message);
  }
}
